# Package of request callbacks. The naming convention is 
#
#    <HTTP verb>_<action>
#
# For example, get_index is hit on a GET request, whereas
# post_results would be hit on a POST request.

package Stats::Controller;

use Template::Recall;
use Stats::Util qw(scrape_data get_stats commify);
use strict;
use warnings;

## actions: HTTP verb + URI
## For this example, there is only one request handler: get_index
# GET /
sub get_index {
    my ($class, $req, $p) = @_;

    my $file_name = generate_report();
    my $page = load_page($file_name);
    my $res;

    if ($page) {
	$res = generate_response($req, 200, 'text/html');
	$res->content($page);
    }
    else {
	$res = generate_response($req, 404, 'text/plain');
	$res->content('Not found');
    }
    return $res;
}

## helpers
sub load_page {
    my $file_name = shift;
    my $page = '';

    open(INFILE, "<$file_name");
    while(<INFILE>) {
	my ($line) = $_;
	$page .= $line;
    }
    close(INFILE);
    return $page;
}

sub generate_report {
    my $basic_data = scrape_data();
    my %data_hash = %$basic_data;

    my $outfile = 'reportFinal.html';
    open(OUTFILE, ">$outfile");

    my $tr = Template::Recall->new(template_path => 'report.html');
    print OUTFILE $tr->render('header');
    print OUTFILE $tr->render('banner');
    print OUTFILE $tr->render('table1');

    ## Generate the table's rows.
    my $odd_even = 0;
    my @keys = sort keys %data_hash;

    # Basic data on major and income.
    my $key;
    foreach $key (@keys) {
	my $values_ref = $data_hash{$key};
	my @values = @$values_ref;

	my %row = (
	    major => $key,
	    wage  => '$' . commify($values[0]),
	    p25   => '$' . commify($values[1]),
	    p75   => '$' . commify($values[2]),
	    pop   => $values[3],
	    grad  => $values[4],
	    boost => $values[5]
	    );
	if ($odd_even % 2) {
	    print OUTFILE $tr->render('odd', \%row);
	}
	else {
	    print OUTFILE $tr->render('even', \%row);
	}
	$odd_even += 1;
    }
    print OUTFILE $tr->render('end1');

    # Statistical data on mean wages per study area.
    my $stats_ref = get_stats();
    my @stats_list = @$stats_ref;
    my %stats = (
	min    => $stats_list[0],
	max    => $stats_list[1],
	range  => $stats_list[2],
	meanA  => $stats_list[3],
	meanG  => $stats_list[4]
	);
    print OUTFILE $tr->render('table2', \%stats);
    print OUTFILE $tr->render('end2');
    close(OUTFILE);        
    return $outfile;
}

sub generate_response {
    my ($req, $code, $mime) = @_;
    my $res = $req->new_response($code);
    $res->content_type($mime);
    return $res;
}

1;

