use strict;
use lib "$ENV{HOME}/scrape/lib";  ### May need adjustment on your local system

use Plack::Request;
use Plack::Builder;
use HTTP::Router::Declare;
use Stats::Controller;

# Router patterns
#
# Match the request path to a controller/action, where the
# controller is the module Stats::Controller and the
# action is one of the functions defined therein.
my $router = router {
    # Get the raw data, then generate the report: get_index is the function
    match '/', {method => 'GET'}, 
    to {controller => 'Controller', action => 'index'};
    # Other actions as needed
};

# Application object: 
#
#   -- map the request path to the appropriate controller/action
#      or return a 404 (Not Found) error if there is no match
#   
#   -- invoke the controller/action to generate the HTTP response
#
#   -- send the response to the client
#
my $app = sub {
    my $request = Plack::Request->new(shift);

    my $match = $router->match($request)
        or return $request->new_response(404)->finalize;

    my $params = $match->params;
    my $controller = 'Stats::' . $params->{controller};

    my $action = $controller->can(lc($request->method) . "_" . $params->{action})
        or return $request->new_response(405)->finalize;

    my $response = $controller->$action($request, $params);
    $response->finalize;
};

print "Process id for killing the server is: $$\n";
return $app; 
